#!/usr/bin/env python
# -*- coding: utf-8 -*-
# developed & tested with Python 3.7

import time
from datetime import datetime, timedelta, timezone
from Gpib import *
from decimal import *
from HPAK_3458A import *
from efloat import *

measurement_filename = ""

filename = "cal_testing_22.csv"
gpib_address = 22

acal_duration = timedelta(seconds=960)	# DCV = 180s, OHMS = 900s, ALL = 960s
acal_interval = timedelta(hours=1)
acal_earliest_finish_time = datetime.now(timezone.utc) + acal_duration
acal_execution_time = (acal_earliest_finish_time + timedelta(hours=1)).replace(microsecond=0, second=0, minute=0) - acal_duration
#acal_execution_time = acal_earliest_finish_time.replace(microsecond=0, second=0, minute=57)
	
delay_read_measurement = 0.0
max_delay = 27.3 #0.5 #4.15s @ 100NPLC, OHMF 27,25s

def setup_dmm(addr):
    inst = Gpib(0, addr)
    inst.timeout(gpib.T1000s)
    inst.clear()
    inst.write("PRESET NORM")
    inst.write("OFORMAT ASCII")
    #inst.write("OHMF 1E4")				#10k 4Wire
    #inst.write("OCOMP ON")				# ON = offset compensation for Resistance measurements
    #inst.write("DELAY 1")				# delay in s for resistance measurements with OCOMP ON
    inst.write("DCV 10")				#10V AUTO
    #inst.write("AZERO OFF")			####### 2,11s@100plc
    #inst.write("RATIO ON")
    inst.write("TARM HOLD")
    inst.write("TRIG AUTO")
    inst.write("NPLC 100")
    #inst.write("APER 1")				# For precision 4-wire
    inst.write("FIXEDZ OFF")			# ON = 10Meg on all ranges
    inst.write("NRDGS 1,AUTO")			# LINE
    inst.write("MEM OFF")
    inst.write("END ALWAYS")
    inst.write("NDIG 9")
    inst.write("DISP MSG,\"                 \"")
    inst.write("TARM AUTO")
    # inst.write("TRIG LINE")
    # inst.write("DISP OFF")
    return inst

def run_acal(inst):
    #inst.write("ACAL DCV")
    #inst.write("ACAL OHMS")
    inst.write("ACAL ALL")

def record_cal_values(inst, filename):
    # try:
        global acal_execution_time
        global acal_duration
        
        test_time = (acal_execution_time + acal_duration).replace(tzinfo=timezone.utc).astimezone(tz=None).isoformat()
        
        inst.write("CAL? 175")
        temp = inst.read().strip().decode('utf-8')
        
        # Read ACAL DCV 10V zero front
        inst.write("CAL? 7")
        cal_dcv_zero = inst.read().strip().decode('utf-8')
        
        # Read ACAL DCV 10V Gain value
        inst.write("CAL? 72")
        cal_dcv_gain = inst.read().strip().decode('utf-8')
        
        # Read 40K reference actual value
        inst.write("CAL? 1,1")
        cal_40k_actual = inst.read().strip().decode('utf-8')
        
        # Read 7V reference actual value
        inst.write("CAL? 2,1")
        cal_7v_actual = inst.read().strip().decode('utf-8')

        # Clear Display
        inst.write("DISP MSG,\"                 \"")
        
        print ("{} > Temp: {} Zero: {} Gain: {} 40k: {} 7V: {}".format(test_time, temp, cal_dcv_zero, cal_dcv_gain, cal_40k_actual, cal_7v_actual))
        
        values = map(str, [test_time, temp, cal_dcv_zero, cal_dcv_gain, cal_40k_actual, cal_7v_actual])
        with open(filename, 'a') as file_handle:
            file_handle.write(";".join(values) + '\n')

        record_all_cal_values(inst)
    # except:
        # pass

def record_all_cal_values(inst, filename="3458A CAL values.csv"):
	try:
		global acal_execution_time
		global acal_duration
	
		test_time = (acal_execution_time + acal_duration).replace(tzinfo=timezone.utc).astimezone(tz=None).isoformat() + 'Z'

		all_cal_values = dict()
		all_cal_values[0] = test_time

		acq_start_time = datetime.now(timezone.utc).replace(microsecond=0).astimezone(tz=None).replace(tzinfo=None)
		print("{} > reading all cal values".format(acq_start_time))

		for i in range(1, 253):
			inst.write("CAL? %d" % i)
			all_cal_values[i] = inst.read().strip().decode('utf-8')

		acq_stop_time = datetime.now(timezone.utc).replace(microsecond=0).astimezone(tz=None).replace(tzinfo=None)
		diff_time = acq_stop_time - acq_start_time
		print("{} > finished in {}s".format(acq_stop_time, diff_time.total_seconds()))

		# defeats
		inst.write("CAL? 2437")
		all_cal_values[2437] = inst.read().strip().decode('utf-8')

		# destructive overloads
		inst.write("CAL? 2941")
		all_cal_values[2941] = inst.read().strip().decode('utf-8')

		# print (all_cal_values)
		with open(filename, 'a') as file_handle:
			file_handle.write(';'.join(all_cal_values.values()) + '\n')
	
	except:
		pass

def run_measurement(inst, filename):
	global delay_read_measurement
	global max_delay
	
	target_read_measurement_time = 0.01
	
	start = time.time()

	data = inst.read().strip().decode('utf-8')

	end = time.time()

	test_time = datetime.now(timezone.utc).replace(microsecond=0)

	time_elapsed_read_measurement = end - start

	# Get temperature
	#inst.write("TEMP?")
	#temp = inst.read().strip().decode('utf-8')
	temp = 0

	# Get current function & range
	inst.write("FUNC?")
	function_range = inst.read().strip().decode('utf-8').split(', ')
	function = HPAK_3458A.FUNCTION[function_range[0]]
	range = function_range[1]

	# convert to naive local time (without timezone)
	acq_time = test_time.astimezone(tz=None).replace(tzinfo=None)

	EFloat.precision = 9
	EFloat.exponent_letter = 'E'
	value = EFloat(data)
	#value = Decimal(data)
	#value.normalize
	print ("{} > {} {} {} (read: {:.3f}s, delay: {:.3f}s)".format(acq_time, value, function, range, time_elapsed_read_measurement, delay_read_measurement))
	
	# Save values
	#values = map(str, [test_time.astimezone().isoformat(), data, function, range, temp])

	#with open(filename, 'a') as file_handle:
	#	file_handle.write(";".join(values) + '\n')

	delay_read_measurement = delay_read_measurement + time_elapsed_read_measurement - target_read_measurement_time
	if delay_read_measurement < target_read_measurement_time:
		delay_read_measurement = time_elapsed_read_measurement
	if delay_read_measurement > max_delay:
		delay_read_measurement = max_delay
	time.sleep(delay_read_measurement)

    # except:
		# print 'ex'
		# pass
	
		
def main():
	global delay_read_measurement
	global acal_duration
	global acal_interval
	global acal_execution_time
	global filename
	global measurement_filename
	global gpib_address
	
	dmm1 = setup_dmm(gpib_address)
	
	while True:
		print ("Collect Data DMM {} to \"{}\"".format(gpib_address, measurement_filename))
		print ("Next ACAL at {}".format(acal_execution_time.replace(tzinfo=timezone.utc).astimezone(tz=None)))

		while (datetime.now(timezone.utc) < acal_execution_time - timedelta(0,delay_read_measurement)):
		#while (True):
			run_measurement(dmm1, measurement_filename)
		
		actual_time = datetime.now(timezone.utc)
		if acal_execution_time > actual_time:
			print ("Start ACAL in {}s".format((acal_execution_time - actual_time).seconds))
			time.sleep((acal_execution_time - actual_time).seconds)
		
		print ("ACAL DCV on DMM {}".format(gpib_address))
		run_acal(dmm1)
		
		# wait till ACAL finishes
		acal_sleep = (acal_execution_time - datetime.now(timezone.utc) + acal_duration).seconds
		print ("ACAL finishes at {} in {}s".format((datetime.now(None) + timedelta(seconds=acal_sleep)).replace(microsecond=0), acal_sleep))
		time.sleep(acal_sleep)

		record_cal_values(dmm1, filename)

		acal_execution_time += acal_interval
		delay_read_measurement = 0
		
		
	print ("Done with measurements")

if __name__ == "__main__":
	main()